@@ -12,7 +12,7 @@ from pathlib import Path
 from pycaret.classification import *
 from sklearn.metrics import recall_score
 from mlflow.models.signature import infer_signature
-
+from explainerdashboard import ExplainerDashboard, ClassifierExplainer
 
 SENSORS_FEATURES = [
     "sensor_1",
@@ -96,10 +96,27 @@ def specificity(actual, pred, pos_label=0):
 
 def objective(trial, df, study_name, model_mode):
 
+
+
     mlflow.set_experiment(study_name)
 
     mlflow.set_tracking_uri(REMOTE_TRACKING_URI)
 
+
+    df_offset = df.filter(regex='offset')
+    df_gradient = df.filter(regex='gradient')
+
+    if len(df_offset.columns) > 0:
+        offset = True
+    else:
+        offset = False
+
+    if len(df_gradient.columns) > 0:
+        gradient = True
+    else:
+        gradient = False
+    
+
     df_copy = df.copy(deep=True)
 
     relevant_features = list(df_copy.drop(TARGET_COL, axis=1).columns)
@@ -119,6 +136,8 @@ def objective(trial, df, study_name, model_mode):
         log_plots=True,
         experiment_name=study_name,
         preprocess=False,
+        remove_multicollinearity=True,
+        multicollinearity_threshold=1.0
     )
 
     add_metric("specificity", "Specificity", specificity)
@@ -173,6 +192,30 @@ def objective(trial, df, study_name, model_mode):
 
     model_params = model.get_params()
 
+
+
+    dashboard_dir = os.path.join(os.getcwd(), "dashboard")
+    Path(dashboard_dir).mkdir(parents=True, exist_ok=True)
+
+    X_test = get_config('X_test')
+    y_test = get_config('y_test')
+
+    explainer = ClassifierExplainer(model, X_test, y_test)
+    db = ExplainerDashboard(explainer, 
+                         # defaults to "Model Explainer"
+                        whatif=False,
+                        shap_interaction=False
+                        )
+
+    db.save_html(os.path.join(dashboard_dir, 'dash.html'))
+
+    mlflow.log_artifact(dashboard_dir)
+
+    
+
+
+
+
     mlflow.log_metrics(metrics)
 
     try:
@@ -183,8 +226,11 @@ def objective(trial, df, study_name, model_mode):
         pass
 
     mlflow.set_tag("model_name", model_name)
+    mlflow.set_tag("offset", offset)
+    mlflow.set_tag("gradient", gradient)
     mlflow.set_tag("model_params", model_params)
 
+
     mlflow.set_tag("model_mode", model_mode)
 
     mlflow.end_run()
